//////////////////////////////////////////////////////////////////////////////
//
// crc calculation
// This VERILOG code was generated using CRCGEN.PL version 1.7
// Last Modified: 01/02/2002
// Options Used:
//    Module Name = crc32
//      CRC Width = 12
//     Data Width = 4
//     CRC Init   = 0
//     Polynomial = [0 -> 12]
//        1 1 1 1 0 0 0 0 0 0 0 1 1
//
// Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//             WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//             IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//             A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Copyright (c) 2001,2002 Xilinx, Inc.  All rights reserved.
//
//
//////////////////////////////////////////////////////////////////////////////

module crc32 (
   crc_reg, 
   crc,
   d,
   calc,
   init,
   d_valid,
   clk,
   reset
   );

output [11:0] crc_reg;
output [3:0]  crc;

input  [3:0]  d;
input         calc;
input         init;
input         d_valid;
input         clk;
input         reset;

reg    [11:0] crc_reg;
reg    [3:0]  crc;

//////////////////////////////////////////////////////////////////////////////
// Internal Signals
//////////////////////////////////////////////////////////////////////////////
wire   [11:0] next_crc;

//////////////////////////////////////////////////////////////////////////////
// Infer CRC-12 registers
// 
// The crc_reg register stores the CRC-12 value.
// The crc register is the most significant 4 bits of the 
// CRC-12 value.
//
// Truth Table:
// -----+---------+----------+----------------------------------------------
// calc | d_valid | crc_reg  | crc 
// -----+---------+----------+----------------------------------------------
//  0   |     0   | crc_reg  | crc 
//  0   |     1   |  shift   | bit-swapped, complimented msbyte of crc_reg
//  1   |     0   | crc_reg  | crc 
//  1   |     1   | next_crc | bit-swapped, complimented msbyte of next_crc
// -----+---------+----------+----------------------------------------------
// 
//////////////////////////////////////////////////////////////////////////////
 
always @ (posedge clk or posedge reset)
begin
   if (reset) begin
      crc_reg <= 12'h000;
      crc     <= 4'h0;
   end
   
   else if (init) begin
      crc_reg <= 12'h000;
      crc     <=  4'h0;
   end

   else if (calc & d_valid) begin
      crc_reg <= next_crc;
      crc     <= ~{next_crc[8], next_crc[9], next_crc[10], next_crc[11]};
   end
   
   else if (~calc & d_valid) begin
      crc_reg <=  {crc_reg[7:0], 4'h0};
      crc     <= ~{crc_reg[4], crc_reg[5], crc_reg[6], crc_reg[7]};
   end
end

//////////////////////////////////////////////////////////////////////////////
// CRC XOR equations
//////////////////////////////////////////////////////////////////////////////

assign next_crc[0] = d[0] ^ crc_reg[9] ^ d[1] ^ crc_reg[10] ^ d[2] ^ crc_reg[11] ^ d[3] ^ crc_reg[8];
assign next_crc[1] = d[3] ^ crc_reg[8];
assign next_crc[2] = d[0] ^ d[1] ^ crc_reg[10] ^ crc_reg[11] ^ d[3] ^ crc_reg[8];
assign next_crc[3] = d[1] ^ crc_reg[10] ^ d[3] ^ crc_reg[8];
assign next_crc[4] = crc_reg[9] ^ d[0] ^ d[2] ^ crc_reg[11] ^ crc_reg[0];
assign next_crc[5] = d[1] ^ crc_reg[10] ^ crc_reg[1];
assign next_crc[6] = d[0] ^ crc_reg[2] ^ crc_reg[11];
assign next_crc[7] = crc_reg[3];
assign next_crc[8] = crc_reg[4];
assign next_crc[9] = crc_reg[5];
assign next_crc[10] = crc_reg[6];
assign next_crc[11] = d[0] ^ crc_reg[9] ^ d[1] ^ crc_reg[10] ^ d[2] ^ crc_reg[11] ^ crc_reg[7] ^ d[3] ^ crc_reg[8];
endmodule
